#####################################################################
#####################################################################
#####################################################################
###Source Code of the estimation of model (9) in the paper
###Estimating daily climatological normals in a changing climate
#####################################################################
#####################################################################
#####################################################################


#a naive  
#implementation of the estimation, more efficient coding
#involved a precalculation of the correspondance between 
#smoothing parameter degree of freedom, gram matrix, spline basis
#and was not easly transferable to other timesteps

#Estimation of the function f,g,h of model (9)
##Main function :
#### input : 
# - Y : The considered time serie in a matrix form
#         - lines representing the years
#         - colums the seasonality
# - df, dg, dh : degrees of freedom
#-years : the years in witch the estimation takes places (in the order of the marrix Y)
#-seasonality : the labels/names of the seasonal components
#### output : 
##the estimated functions f,g,h of model 9 :fhat ,ghat ,hhat
#years corresponding to the training set
#


NormNStaFun=function(Y,df=10, dg=6, dh=10,years=NULL){

p = dim(Y)[2]
n = dim(Y)[1]
if(is.null(years)){
xn = 1:n
}else{
  xn=years
}

xp = t(1:p)-1
## matrice de lissage

## estimation of the annual trend

moyAnnee=apply(Y,1,mean)

s  <- smooth.spline(xn,moyAnnee,df = dg)


fnoise=NULL
hnoise=NULL

for (d in 1:365){
  tempFrame=matrix(c(Y[,d],predict(s, xn)$y-mean(predict(s, xn)$y)),ncol = 2,nrow = length(xn))
  nom=c("expliquee","g")
  colnames(tempFrame)=nom
  lm.fh=lm(expliquee~.,data=as.data.frame(tempFrame))
  fnoise=c(fnoise,lm.fh$coefficients[1])
  hnoise=c(hnoise,lm.fh$coefficients[2])
}

##### Calculation of the smoothing matrix for periodic splines
#------------------------------------------------------------
#spline basis at the design points Zp and Gram matrix Gp
source("baseSplinePer.R",local=T);
#function estimating the smoothing parameter corresponding the degree of freedom
source("Dl2rho.R",local=T);

rho_pf=dlrho(dl = df,letter = "p")
rho_ph=dlrho(dl = dh,letter = "p")



## H, Gamma for f:
 Hp = t(Zp)%*%Zp + rho_pf * Gp
 Hp1 = solve(Hp)
 Gamma_pf = Zp %*% Hp1 %*% t(Zp)
 ## H, Gamma for h: 
 Hp = t(Zp)%*%Zp + rho_ph * Gp
 Hp1 = solve(Hp)
 Gamma_ph = Zp %*% Hp1 %*% t(Zp)

#plot(Gamma_pf%*%f,type='l',col=2)
#points(f)

#plot(Gamma_ph%*%h,type='l',col=2)
#points(h)

#estimation of the periodic splines
s.f <- Gamma_pf%*%fnoise
s.h =Gamma_ph%*%hnoise
s.g=predict(s, xn)$y-mean(predict(s, xn)$y)
return(list(fhat=s.f,ghat=s.g,hhat=s.h,years=xn))

}

####################################################################
#function permitting to predict a given year once the 
#function f,g,h have been estimated

#### input : 
# -fhat ghat hhat : trace of the estimated 
# -years : years on which the training occured
# -new.years : the years in witch the prediction takes places 

#### output : 
# -Yhat : prediction on the considered years in a matrix form
#         - lines representing the years
#         - colums the seasonality



predict.NormNSta=function(fhat=NULL,ghat=NULL,hhat=NULL,years=NULL,new.years=NULL
                          ,fghHatYears=NULL){
  
  if(!is.null(fghHatYears)){
    fhat=fghHatYears$fhat
    ghat=fghHatYears$ghat
    hhat=fghHatYears$hhat
    years=fghHatYears$years
  }
  s.ghat=splinefun(x=years,ghat,method = "natural")
  
  if(is.null(new.years)){
    new.years=years
  }
  
  nyears=length(new.years)
  
  Yhat=rep(fhat,times=nyears)+rep(s.ghat(new.years),each=length(fhat))*rep(hhat,times=nyears)
  Yhat=matrix(data = Yhat,nrow =nyears ,byrow = T)
  row.names(Yhat)=new.years
  return(Yhat)
  }


